Imports System.ComponentModel.DataAnnotations.Schema
Imports CommonData

Namespace SalesDomain
  Public Class Order
	Public Shared Function CreateOrder(ByVal customer As CustomerForOrderTransferDTO) As Order
	  Return New Order(customer)
	End Function

	Private Sub New(ByVal customer As CustomerForOrderTransferDTO)

            LineItems = New List(Of LineItem)
	  CustomerId = customer.CustomerId
	  OrderDate = Date.Now
	  ModifiedDate = Date.Now
	  SetShippingAddress(customer.ShippingAddress)
	  Select Case customer.Status
		Case CustomerStatus.Silver
		  CustomerDiscount = 0
		Case CustomerStatus.Gold
		  CustomerDiscount =.05
		Case CustomerStatus.Platinum
		  CustomerDiscount =.1
	  End Select
	End Sub
	Private Sub New()
            LineItems = New List(Of LineItem)
	End Sub

        Private PrivateOrderId As Integer
        Public Property OrderId As Integer
            Get
                Return PrivateOrderId
            End Get
            Private Set(ByVal value As Integer)
                PrivateOrderId = value
            End Set
        End Property
        Private PrivateOrderDate As Date
        Public Property OrderDate As Date
            Get
                Return PrivateOrderDate
            End Get
            Private Set(ByVal value As Date)
                PrivateOrderDate = value
            End Set
        End Property
        Private PrivateDueDate? As Date
        Public Property DueDate As Date?
            Get
                Return PrivateDueDate
            End Get
            Private Set(ByVal value As Date?)
                PrivateDueDate = value
            End Set
        End Property
        Private PrivateOnlineOrder As Boolean
        Public Property OnlineOrder As Boolean
            Get
                Return PrivateOnlineOrder
            End Get
            Private Set(ByVal value As Boolean)
                PrivateOnlineOrder = value
            End Set
        End Property
        Private PrivatePurchaseOrderNumber As String
        Public Property PurchaseOrderNumber As String
            Get
                Return PrivatePurchaseOrderNumber
            End Get
            Private Set(ByVal value As String)
                PrivatePurchaseOrderNumber = value
            End Set
        End Property
        Private PrivateComment As String
        Public Property Comment As String
            Get
                Return PrivateComment
            End Get
            Private Set(ByVal value As String)
                PrivateComment = value
            End Set
        End Property
        Private PrivatePromotionId As Integer
        Public Property PromotionId As Integer
            Get
                Return PrivatePromotionId
            End Get
            Private Set(ByVal value As Integer)
                PrivatePromotionId = value
            End Set
        End Property
	'persisting the address used for this order in case
	'customer.shippingAddress changes in the future
        Private PrivateShippingAddress As Address
        Public Property ShippingAddress As Address
            Get
                Return PrivateShippingAddress
            End Get
            Private Set(ByVal value As Address)
                PrivateShippingAddress = value
            End Set
        End Property

	Public Sub SetShippingAddress(ByVal address As Address)
	  ShippingAddress = New Address(address.Street1, address.Street2, address.City, address.Region, address.Country, address.PostalCode)
	End Sub


        Public ReadOnly Property Discount As Double
            Get
                Return CustomerDiscount + PromoDiscount
            End Get
        End Property

        Private PrivateCustomerDiscount As Double
        Public Property CustomerDiscount As Double
            Get
                Return PrivateCustomerDiscount
            End Get
            Private Set(ByVal value As Double)
                PrivateCustomerDiscount = value
            End Set
        End Property
        Private PrivatePromoDiscount As Double
        Public Property PromoDiscount As Double
            Get
                Return PrivatePromoDiscount
            End Get
            Private Set(ByVal value As Double)
                PrivatePromoDiscount = value
            End Set
        End Property


	Public Sub SetOrderDetails(ByVal onLineOrder As Boolean, ByVal PONumber As String, ByVal comment As String, ByVal promotion As Promotion)
	  Me.OnlineOrder = onLineOrder
	  PurchaseOrderNumber = PONumber
	  Me.Comment = comment
	  If promotion IsNot Nothing Then
		PromotionId = promotion.PromotionId
		PromoDiscount = promotion.Discount
	  End If
	End Sub

        Private PrivateSalesOrderNumber As String
        Public Property SalesOrderNumber As String
            Get
                Return PrivateSalesOrderNumber
            End Get
            Private Set(ByVal value As String)
                PrivateSalesOrderNumber = value
            End Set
        End Property

        Private PrivateCustomerId As Integer
        Public Property CustomerId As Integer
            Get
                Return PrivateCustomerId
            End Get
            Private Set(ByVal value As Integer)
                PrivateCustomerId = value
            End Set
        End Property
        Private PrivateSubTotal As Double
        Public Property SubTotal As Double
            Get
                Return PrivateSubTotal
            End Get
            Private Set(ByVal value As Double)
                PrivateSubTotal = value
            End Set
        End Property
        Private PrivateModifiedDate As Date
        Public Property ModifiedDate As Date
            Get
                Return PrivateModifiedDate
            End Get
            Private Set(ByVal value As Date)
                PrivateModifiedDate = value
            End Set
        End Property
        Private PrivateLineItems As ICollection(Of LineItem)
        Public Property LineItems As ICollection(Of LineItem)
            Get
                Return PrivateLineItems
            End Get
            Private Set(ByVal value As ICollection(Of LineItem))
                PrivateLineItems = value
            End Set
        End Property
Public Sub CreateLineItem(ByVal product As Product, ByVal quantity As Integer)
  'NOTE: add some validations such as:
  'quantity can't be 0 and if it's >1000 give a discount
  ' customer is a gold. give another discount
            Dim item = New LineItem(OrderId, product.ProductId, quantity, product.ListPrice, CustomerDiscount + PromoDiscount)
  LineItems.Add(item)
End Sub


  End Class
End Namespace